@@ -6,7 +6,7 @@
 //   Larry Ewing <lewing@novell.com>
 //   Gabriel Burt <gburt@novell.com>
 //
-// Copyright (C) 2008 Novell, Inc.
+// Copyright 2008-2010 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -124,8 +124,8 @@ namespace Banshee.NowPlaying
             Gdk.Screen screen = Screen;
             int monitor = screen.GetMonitorAtWindow (parent.GdkWindow);
             Gdk.Rectangle bounds = screen.GetMonitorGeometry (monitor);
-            Move (bounds.X, 0);
-            SetDefaultSize (bounds.Width, bounds.Height);
+            Move (bounds.X, bounds.Y);
+            Resize (bounds.Width, bounds.Height);
         }
 
         protected override void OnRealized ()
@@ -134,6 +134,8 @@ namespace Banshee.NowPlaying
 
             base.OnRealized ();
 
+            GdkWindow.OverrideRedirect = true;
+
             Screen.SizeChanged += OnScreenSizeChanged;
         }
 
